EMAIL_TEMPLATES = {
    'user_invitation': 'argomi_invitation.html',
    'user_welcome': 'argomi_welcome.html',
    'user_signup': 'argomi_signup.html',
    'user_change_email': 'argomi_change_email.html',
    'forgot_password': 'argomi_change_password.html',
    'default_message': 'argomi_default_message.html'
}